;////////////////////////////////////////////////////////////////////////////;
;/ kbd.s                                                                    /;
;////////////////////////////////////////////////////////////////////////////;

;////////////////////////////////////////////////////////////////////////////;
;/ includes                                                                 /;
;////////////////////////////////////////////////////////////////////////////;
	.include "\dev\gb\lib\global.s"

;////////////////////////////////////////////////////////////////////////////;
	.area	_HEADER (ABS)
;////////////////////////////////////////////////////////////////////////////;
	.org	0x58		;SIO (overrides GBDK irq)
	jp	.Keyboard_irq

;////////////////////////////////////////////////////////////////////////////;
	.area	_BSS
;////////////////////////////////////////////////////////////////////////////;
.kbd_handler_jp:		;jump instruction
	.ds 1
.kbd_handler:			;scancode handler
	.ds 2
.release:				;release adds 0x80 to scancode
	.ds 1
_kbd_errors::			;keyboard errors detected
	.ds 1

;////////////////////////////////////////////////////////////////////////////;
	.area	_DATA
;////////////////////////////////////////////////////////////////////////////;
.Scancodes:
	.include "kbdtable.s"

;////////////////////////////////////////////////////////////////////////////;
	.area	_CODE
;////////////////////////////////////////////////////////////////////////////;

;////////////////////////////////////////////////////////////////////////////;
;/ _InitKeyboard                                                            /;
;////////////////////////////////////////////////////////////////////////////;
_InitKeyboard::
	ld	a, #0xC3		;place jump
	ld	(.kbd_handler_jp), a
	
	lda	hl, 2(sp)		;set handler pointer
	ld	a, (hl+)
	ld	(.kbd_handler), a
	ld	a, (hl)
	ld	(.kbd_handler+1), a
	
	xor	a, a
	ld	(.release), a
	ld	(_kbd_errors), a

	ldh	a, (.IE)		;enable SIO interrupt bit
	or	a, #.SIO_IFLAG
	ldh	(.IE), a
	
	ld	a, #0x80		;start external transfer
	ldh	(.SC), a
	
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ .Keyboard_irq                                                            /;
;////////////////////////////////////////////////////////////////////////////;
.Keyboard_irq:
	ei					;make other interrupts available
	
	push af
	push bc
	
	ldh	a, (.SB)		;s0123456 (startbit + 7 databits)
	ld	b, a
	ld	a, #0x80		;start reading remaining bits
	ldh	(.SC), a
	
	ld	c, #48			;delay for getting the 3 other bits
	ld	a, (__cpu)
	cp	a, #0x11		;GBC?
	jr	nz, .loop1
	ld	a, (.key1)
	and	a, #0x80		;double speed?
	jr	z, .loop1
	sla	c				;double the delay

.loop1:
	dec	c
	jr	nz, .loop1		;wait for the 3 other bits
	
	xor	a, a			;cancel transfer of second byte
	ldh	(.SC), a		;	(I don't know if this does anything, but...)
	ldh	a, (.SB)		;234567ps (6 databit + paritybit + stopbit)
	
	srl a				;_234567p (stopbit to carry)
	jr	nc, .fail		;stopbit != 1 ?
	sla	b				;0123456_ (startbit to carry)
	jr	c, .fail		;startbit != 0 ?
	
	push de
	push hl
	
	srl	a				;__234567 (parity to carry)
	rr	e				;p??????? (parity stored in bit 7 of e)
	or	a, b			;01234567
	
	ld	b, #0			;look up scancode...
	ld	c, a
	ld	hl, #.Scancodes
	add	hl, bc
	
	ld	a, (hl)			;psssssss
	xor	a, e			;1???????
	bit	7, a
	jr	z, .fail_hlde	;parity error?
	
	ld	a, (hl)			;psssssss
	and	a, #0x7F		;0sssssss
	jr	z, .fail_hlde	;scancode 0 does not exist
	
	cp	a, #0x60		;extended code 1 ?
	jr	z, .call_handler_bit7set
	cp	a, #0x61		;extended code 2 ?
	jr	z, .call_handler_bit7set
	
	ld	hl, #.release	;release flag address
	cp	a, #0x7F		;release code ? (defined in KBDTABLE.C)
	jr	nz, .dont_set_release
	inc	a
	ld	(hl), a			;10000000 (set release flag)
	jr	.done_hlde
.dont_set_release:
	or	a, (hl)			;rsssssss
	ld	(hl), b			;00000000
	
.call_handler:
	dec	sp
	lda	hl, 0(sp)
	ld	(hl), a			;scancode on stack
	ld	hl, #.kbd_handler_jp
	call .jp_hl
	inc	sp

.done_hlde:
	pop hl
	pop de
.done:
	ld	a, #0x80		;start external transfer again
	ldh	(.SC), a
	
	pop bc
	pop	af
	reti

.fail_hlde:
	pop hl
	pop de
.fail:
	xor	a, a			;clear release flag
	ld	(.release), a
	
	ld	hl, #_kbd_errors
	inc	(hl)
	
	jr	.done

.jp_hl:
	jp	(hl)

.call_handler_bit7set:
	or	a, #0x80
	jr	.call_handler
