//////////////////////////////////////////////////////////////////////////////
// kbdtable.cpp                                                             //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include "kbdtable.h"

//////////////////////////////////////////////////////////////////////////////
// defines                                                                  //
//////////////////////////////////////////////////////////////////////////////
#define RELEASED 0x7F // just an unused value

//////////////////////////////////////////////////////////////////////////////
// variables                                                                //
//////////////////////////////////////////////////////////////////////////////
unsigned char scancodes[256] = // AT-keyboard (Windows 95)
{
// 0x00
	0,
	K_F9,
	0,
	K_F5,
	K_F3,
	K_F1,
	K_F2,
	K_F12,
	0,
	K_F10,
	K_F8,
	K_F6,
	K_F4,
	K_TAB,
	K_BACKQUOTE,
	0,
// 0x10
	0,
	K_ALT,
	K_LSHIFT,
	0,
	K_CONTROL,
	K_Q,
	K_1,
	0,
	0,
	0,
	K_Z,
	K_S,
	K_A,
	K_W,
	K_2,
	K_LSTART,
// 0x20
	0,
	K_C,
	K_X,
	K_D,
	K_E,
	K_4,
	K_3,
	K_RSTART,
	0,
	K_SPACE,
	K_V,
	K_F,
	K_T,
	K_R,
	K_5,
	K_POPUP,
// 0x30
	0,
	K_N,
	K_B,
	K_H,
	K_G,
	K_Y,
	K_6,
	0,
	0,
	0,
	K_M,
	K_J,
	K_U,
	K_7,
	K_8,
	0,
// 0x40
	0,
	K_COMMA,
	K_K,
	K_I,
	K_O,
	K_0,
	K_9,
	0,
	0,
	K_POINT,
	K_SLASH,
	K_L,
	K_SEMICOLON,
	K_P,
	K_MINUS,
	0,
// 0x50
	0,
	0,
	K_SQUOTE,
	0,
	K_LBRACKET,
	K_EQUAL,
	0,
	0,
	K_CAPSLOCK,
	K_RSHIFT,
	K_ENTER,
	K_RBRACKET,
	0,
	K_BACKSLASH,
	0,
	0,
// 0x60
	0,
	0,
	0,
	0,
	0,
	0,
	K_BACKSPACE,
	0,
	0,
	K_END,
	0,
	K_LEFT,
	K_HOME,
	0,
	0,
	0,
// 0x70
	K_INSERT,
	K_DELETE,
	K_DOWN,
	K_NUM5,
	K_RIGHT,
	K_UP,
	K_ESCAPE,
	K_NUMLOCK,
	K_F11,
	K_PLUS,
	K_PAGEDOWN,
	K_MINUS,
	K_MULTIPLY,
	K_PAGEUP,
	K_SCROLLLOCK,
	0,
// 0x80
	0,
	K_LEFT,
	0,
	K_F7,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
// 0x90
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
// 0xA0
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
// 0xB0
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
// 0xC0
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
// 0xD0
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
// 0xE0
	K_EXTENDED1 & 0x7F,
	K_EXTENDED2 & 0x7F,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
// 0xF0
	RELEASED,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
};

//////////////////////////////////////////////////////////////////////////////
// SWAP                                                                     //
//////////////////////////////////////////////////////////////////////////////
// swaps bit 0-7 to bit 7-0
#define SWAP(n) ( \
	((n)>>0&1)<<7 | \
	((n)>>1&1)<<6 | \
	((n)>>2&1)<<5 | \
	((n)>>3&1)<<4 | \
	((n)>>4&1)<<3 | \
	((n)>>5&1)<<2 | \
	((n)>>6&1)<<1 | \
	((n)>>7&1)<<0 \
)

//////////////////////////////////////////////////////////////////////////////
// PARITY                                                                   //
//////////////////////////////////////////////////////////////////////////////
// calculates parity
#define PARITY(n) (( \
	((n)>>0&1) + \
	((n)>>1&1) + \
	((n)>>2&1) + \
	((n)>>3&1) + \
	((n)>>4&1) + \
	((n)>>5&1) + \
	((n)>>6&1) + \
	((n)>>7&1) \
) & 1)

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
int main()
{
	FILE *f = fopen("kbdtable.s", "wt");
	if (!f) return 1;
	fprintf(f, ";////////////////////////////////////////////////////////////////////////////;\n");
	fprintf(f, ";/ kbdtable.s                                                               /;\n");
	fprintf(f, ";////////////////////////////////////////////////////////////////////////////;\n\n");
	for (int i=0; i<256; i++)
	{
		unsigned char gbkbcode = SWAP(i);
		unsigned char scancode = scancodes[gbkbcode];
		if ((i & 15) == 0) fprintf(f, "\t.db ");
		fprintf(f, "0x%02X", scancode | 0x80*PARITY(gbkbcode));
		if ((i & 15) == 15) fprintf(f, "\n"); else fprintf(f, ",");
	}
	fclose(f);
	return 0;
}
